/*******************************************************************************
 * Copyright (c) 2007, 2010 Association for Decentralized Information Management
 * in Industry THTH ry.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.db.tests.api.request.misc;

import java.util.ArrayList;
import java.util.UUID;
import java.util.concurrent.ConcurrentSkipListSet;

import org.simantics.databoard.Bindings;
import org.simantics.db.Resource;
import org.simantics.db.Session;
import org.simantics.db.WriteOnlyGraph;
import org.simantics.db.common.request.WriteOnlyResultRequest;
import org.simantics.db.exception.DatabaseException;
import org.simantics.layer0.Layer0;

public class SyncAsyncSyncUtils {
	
    private static final int SIZE = (int)1e4;

    public static Resource createNames(Session session, final ConcurrentSkipListSet<String> names, final ArrayList<Resource> resources) throws DatabaseException {

		return session.syncRequest(new WriteOnlyResultRequest<Resource>() {
			
            @Override
            public Resource perform(WriteOnlyGraph g) throws DatabaseException {
                
            	Layer0 b = Layer0.getInstance(g.getService(Session.class));
                
                Resource root = g.newResource();
                g.claim(root, b.InstanceOf, null,  b.Library);
                
                for(int i=0;i<SIZE;i++) {
                    Resource r = g.newResource();
                    g.claim(root, b.ConsistsOf, b.PartOf, r);
                    g.claim(r, b.InstanceOf, null,  b.Library);
                    String name = UUID.randomUUID().toString();
                    g.addLiteral(r, b.HasName, b.NameOf, b.String, name, Bindings.STRING);
                    names.add(name);
                    resources.add(r);
                }
                
                return root;
                
            }
            
		});
    	
    }

}
