/*******************************************************************************
 * Copyright (c) 2007, 2010 Association for Decentralized Information Management
 * in Industry THTH ry.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.db.tests.api.support.transferableGraphSupport;

import java.io.IOException;
import java.util.UUID;

import org.junit.Test;
import org.simantics.databoard.Bindings;
import org.simantics.databoard.binding.Binding;
import org.simantics.databoard.serialization.SerializationException;
import org.simantics.databoard.serialization.Serializer;
import org.simantics.databoard.serialization.SerializerConstructionException;
import org.simantics.db.ReadGraph;
import org.simantics.db.Resource;
import org.simantics.db.Session;
import org.simantics.db.WriteGraph;
import org.simantics.db.common.request.ReadRequest;
import org.simantics.db.common.request.WriteRequest;
import org.simantics.db.exception.DatabaseException;
import org.simantics.db.service.TransferableGraphSupport;
import org.simantics.db.testing.base.ExistingDatabaseTest;
import org.simantics.layer0.Layer0;
import org.simantics.utils.DataContainer;

public class TransferableGraphSupportTest2 extends ExistingDatabaseTest {

	@Test
    public void test() throws Exception {

    	Session session = getSession();

    	final String value = UUID.randomUUID().toString();
        final TransferableGraphSupport support = session.getService(TransferableGraphSupport.class);
        final DataContainer<Resource> resource = new DataContainer<Resource>();

        try {
            session.syncRequest(new WriteRequest() {
                @Override
                public void perform(WriteGraph graph) throws DatabaseException {

                    Layer0 b = Layer0.getInstance(graph);
        			
                    Resource r = graph.newResource();
                    resource.set(r);
                    graph.claim(r, b.InstanceOf, null, b.String);
                    
					try {
	                    Binding binding = Bindings.STRING;
						Serializer serializer = Bindings.getSerializer( binding );
	        			byte[] raw = serializer.serialize(value);
	        			support.setValue(graph, r, null, raw);
					} catch (SerializerConstructionException e) {
						throw new Error(e);
					} catch (SerializationException e) {
						throw new Error(e);
					} catch (IOException e) {
						throw new Error(e);
					}
                }
            });
        } catch (Throwable e) {
        	e.printStackTrace();
            fail("Write transaction threw an unknown exception " + e);
        }
        
        
        try {
            session.syncRequest(new ReadRequest() {
                @Override
                public void run(ReadGraph graph) throws DatabaseException {
                    String v = graph.getValue(resource.get());
                    assert(v.equals(value));
                }
            });
        } catch (Throwable e) {
            fail("Write transaction threw an unknown exception " + e);
        }
        
    }

}
