/*******************************************************************************
 * Copyright (c) 2007, 2010 Association for Decentralized Information Management
 * in Industry THTH ry.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.db.tests.api.support.undoRedoSupport;

import org.junit.Test;
import org.simantics.databoard.Bindings;
import org.simantics.db.ReadGraph;
import org.simantics.db.Resource;
import org.simantics.db.Session;
import org.simantics.db.WriteGraph;
import org.simantics.db.common.request.ReadRequest;
import org.simantics.db.common.request.WriteRequest;
import org.simantics.db.exception.DatabaseException;
import org.simantics.db.exception.NoSingleResultException;
import org.simantics.db.service.UndoRedoSupport;
import org.simantics.db.testing.base.ExistingDatabaseTest;

/**
 * Tests cases where a non-functional relation uses
 * {@link ReadGraph#getSingleObject(org.simantics.db.Resource, org.simantics.db.Resource)}
 * with > 1 objects.
 * 
 * <p>
 * Should throw {@link NoSingleResultException}, at the time of writing this
 * test, AsyncBarrierImpl refcounting is trashed and DB client is stuck.
 */
public class RedoTest2 extends ExistingDatabaseTest {

	@Test
    public void testRedo2() throws DatabaseException {

    	Session session = getSession();
        final UndoRedoSupport support = session.getService(UndoRedoSupport.class);
        final Resource[] r = new Resource[2];
        try {
            session.syncRequest(new WriteRequest() {
                @Override
                public void perform(WriteGraph g) throws DatabaseException {
                    r[0] = g.newResource();
                    g.claimValue(r[0], "One", Bindings.STRING);
                    g.claimValue(r[0], "Two", Bindings.STRING);
                }
            });
        } catch (Throwable e) {
            fail("Write transaction threw an unknown exception " + e);
        }

        session.syncRequest(new ReadRequest() {
            @Override
            public void run(ReadGraph g) throws DatabaseException {
            	String v = g.getValue(r[0], Bindings.STRING);
            	if (!v.equals("Two"))
                	fail("ClaimValue did not work as expected.");
            }
        });

        support.undo(session, 1);
        support.redo(session, 1);

        session.syncRequest(new ReadRequest() {
            @Override
            public void run(ReadGraph g) throws DatabaseException {
            	String v = g.getValue(r[0], Bindings.STRING);
            	if (!v.equals("Two"))
                	fail("ClaimValue did not work as expected.");
            }
        });
    }
}
