/*******************************************************************************
 * Copyright (c) 2007, 2010 Association for Decentralized Information Management
 * in Industry THTH ry.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.db.tests.api.support.undoRedoSupport;

import org.junit.Test;
import org.simantics.databoard.Bindings;
import org.simantics.db.ReadGraph;
import org.simantics.db.Resource;
import org.simantics.db.Session;
import org.simantics.db.UndoContext;
import org.simantics.db.WriteGraph;
import org.simantics.db.common.UndoContextEx;
import org.simantics.db.common.request.ReadRequest;
import org.simantics.db.common.request.WriteRequest;
import org.simantics.db.exception.DatabaseException;
import org.simantics.db.service.UndoRedoSupport;
import org.simantics.db.testing.base.ExistingDatabaseTest;
import org.simantics.layer0.Layer0;

/**
 * Test case description missing!
 */
public class UndoTest06 extends ExistingDatabaseTest {

	@Test
    public void testUndo6() throws Exception {

    	Session session = getSession();
        final UndoRedoSupport support = session.getService(UndoRedoSupport.class);
        try {
            Context c1 = new Context(session, support);
            c1.create();
            c1.check();
            c1.add();
            c1.check();
            c1.add();
            c1.check();

            c1.undo();
            c1.check();
            c1.undo();
            c1.check();

            c1.add();
            c1.check();
            c1.add();
            c1.check();

            Context c2 = new Context(session, support);
            c2.create();
            c2.check();
            c2.add();
            c2.check();
            c2.add();
            c2.check();
            
//            c1.undo();
//            c1.check();
//            c1.undo();
//            c1.check();
            
            c2.undo();
            c2.check();
            c2.undo();
            c2.check();
        } catch (Throwable t) {
            fail("Test failed with exception " + t);
        }
    }

    class Context {
        private final Session session;
        private final UndoRedoSupport support;
        final UndoContext uctx = new UndoContextEx();
        Resource r;
        Integer value = 0;
        Context(Session session, UndoRedoSupport support) {
            this.session = session;
            this.support = support;
        }
        void create() throws DatabaseException {
            session.syncRequest(new WriteRequest() {
                @Override
                public void perform(WriteGraph graph) throws DatabaseException {
                    graph.markUndoPoint();
                    Layer0 b = Layer0.getInstance(graph);
                    r = graph.newResource();
                    graph.claim(r, b.InstanceOf, b.Entity);
                }
            });
        }
        void add() throws DatabaseException {
            try {
                session.syncRequest(new WriteRequest() {
                    @Override
                    public void perform(WriteGraph graph) throws DatabaseException {
                        graph.markUndoPoint();
                        graph.claimValue(r, ++value, Bindings.INTEGER);
                    }
                });
            } catch (Throwable e) {
                fail("Write transaction threw an unknown exception " + e);
            }
        }
        void check() throws DatabaseException {
            session.syncRequest(new ReadRequest() {
                @Override
                public void run(ReadGraph graph) throws DatabaseException {
                    Integer i = graph.getPossibleValue(r, Bindings.INTEGER);
                    if (null == i) {
                        if (0 != value)
                            fail("Failed to verify value.");
                    } else if (value != i)
                        fail("Failed to verify value expected " + value + " got " + i);
                }
            });
        }
        void undo() throws DatabaseException {
            support.undo(session, 1);
            value -= 1;
        }
    }
}
