/*******************************************************************************
 * Copyright (c) 2007, 2010 Association for Decentralized Information Management
 * in Industry THTH ry.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.db.tests.api.support.undoRedoSupport;

import java.util.ArrayList;
import java.util.List;

import org.junit.Test;
import org.simantics.db.Resource;
import org.simantics.db.Session;
import org.simantics.db.WriteGraph;
import org.simantics.db.common.request.WriteRequest;
import org.simantics.db.common.utils.OrderedSetUtils;
import org.simantics.db.exception.DatabaseException;
import org.simantics.db.service.UndoRedoSupport;
import org.simantics.db.testing.base.ExistingDatabaseTest;

/**
 * Test case description missing!
 */
public class UndoTest10 extends ExistingDatabaseTest {
    
	@Test
    public void testUndo10() throws Exception {

    	Session session = getSession();
        final UndoRedoSupport support = session.getService(UndoRedoSupport.class);
        try {
            UndoTestOrderedSet10 c1 = new UndoTestOrderedSet10(session, support);
            List<Resource> elements = new ArrayList<Resource>(); 
            Resource list = c1.create(elements);
            c1.check(list, elements);
            c1.add(list, elements);
            List<Resource> elements2 = new ArrayList<Resource>();
            elements2.addAll(elements);
            c1.front2(list, elements2);
            c1.check(list, elements2);
            c1.back(list, elements2);
            c1.check(list, elements2);
            c1.front(list, elements2);
            c1.check(list, elements2);
            c1.back(list, elements2);
            c1.check(list, elements2);
            c1.undo();
            c1.undo();
            c1.undo();
            c1.undo();
            c1.check(list, elements);
            c1.add(list, elements);
            elements2.clear();
        } catch (Throwable t) {
            fail("Test failed with exception " + t);
        }
    }
    class UndoTestOrderedSet10 extends UndoTestOrderedSet {
        UndoTestOrderedSet10(Session session, UndoRedoSupport support) {
            super(session, support);
        }
        void front2(final Resource list, final List<Resource> els)
        throws DatabaseException {
            session.syncRequest(new WriteRequest() {
                @Override
                public void perform(WriteGraph g) throws DatabaseException {
                    g.markUndoPoint();
                    FirstLast fl = getFirstLast(g, list, els);
                    if (null == fl.first)
                        return;
                    Resource n = OrderedSetUtils.next(g, list, fl.first);
                    if (n.getResourceId() == fl.first.getResourceId())
                        return;
                    OrderedSetUtils.remove(g, list, n);
                    OrderedSetUtils.addFirst(g, list, n);
                    els.add(0, els.remove(1));
                }
            });
        }
    }
}
