/*******************************************************************************
 * Copyright (c) 2007, 2010 Association for Decentralized Information Management
 * in Industry THTH ry.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.db.tests.api.support.undoRedoSupport;

import org.junit.Test;
import org.simantics.db.ReadGraph;
import org.simantics.db.Resource;
import org.simantics.db.Session;
import org.simantics.db.WriteGraph;
import org.simantics.db.common.request.ReadRequest;
import org.simantics.db.common.request.WriteRequest;
import org.simantics.db.exception.DatabaseException;
import org.simantics.db.exception.NoSingleResultException;
import org.simantics.db.service.UndoRedoSupport;
import org.simantics.db.testing.base.ExistingDatabaseTest;
import org.simantics.utils.DataContainer;

/**
 * Tests cases where a non-functional relation uses
 * {@link ReadGraph#getSingleObject(org.simantics.db.Resource, org.simantics.db.Resource)}
 * with > 1 objects.
 * 
 * <p>
 * Should throw {@link NoSingleResultException}, at the time of writing this
 * test, AsyncBarrierImpl refcounting is trashed and DB client is stuck.
 */
public class UndoTest12 extends ExistingDatabaseTest {

	@Test
    public void testUndo12() throws DatabaseException {

    	Session session = getSession();
        final UndoRedoSupport support = session.getService(UndoRedoSupport.class);
        final DataContainer<Resource> r1 = new DataContainer<Resource>();
        final DataContainer<Resource> r2 = new DataContainer<Resource>();
        try {
            session.syncRequest(new WriteRequest() {
                @Override
                public void perform(WriteGraph g) throws DatabaseException {
                    Resource a = g.newResource();
                    Resource b = g.newResource();
                    r1.set(a);
                    r2.set(b);
                    g.claim(a, a, a);
                    g.deny(a, a, a);
                    g.claim(a, a, b);
                }
            });
        } catch (Throwable e) {
            fail("Write transaction threw an unknown exception " + e);
        }

        check(session, r1.get(), r2.get());

        support.undo(session, 1);
        support.redo(session, 1);

        check(session, r1.get(), r2.get());

    }
    void check(Session session, final Resource a, final Resource b) {
        try {
            session.syncRequest(new ReadRequest() {
                @Override
                public void run(ReadGraph g) throws DatabaseException {
                    if (g.hasStatement(a, a, a))
                        fail("Deny did not work as expected.");
                    if (!g.hasStatement(a, a, b))
                        fail("Add did not work as expected.");
                }
            });
        } catch (Throwable e) {
            fail("Read transaction threw an unexpected exception " + e);
        }
    }
}
