/*******************************************************************************
 * Copyright (c) 2007, 2010 Association for Decentralized Information Management
 * in Industry THTH ry.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.db.tests.api.write.request;

import org.junit.Test;
import org.simantics.db.Resource;
import org.simantics.db.WriteGraph;
import org.simantics.db.common.request.WriteRequest;
import org.simantics.db.exception.CancelTransactionException;
import org.simantics.db.exception.DatabaseException;
import org.simantics.db.testing.base.ExistingDatabaseTest;
import org.simantics.layer0.Layer0;

/**
 * Checks that unexpected write callback failures do not crash the session and
 * that the write request results were correctly committed.
 * 
 * @author Tuukka Lehtonen
 */
public class WriteCancelTest extends ExistingDatabaseTest {

    private static final String AN_ENTITY = "An entity";
    Resource written;
    Resource writtenName;

	@Test
    public void testWriteCancel() throws DatabaseException{

        try {
            getSession().syncRequest(new WriteRequest() {
                @Override
                public void perform(WriteGraph graph) throws DatabaseException {
                    Layer0 b = Layer0.getInstance(graph);
                    written = graph.newResource();
                    writtenName = graph.newResource();
                    graph.claim(written, b.InstanceOf, null, b.Entity);
                    graph.claim(writtenName, b.InstanceOf, null, b.String);
                    graph.claimValue(writtenName, AN_ENTITY);
                    graph.claim(written, b.HasName, writtenName);

                    //throw new DatabaseException("Intentional write request cancellation by unexpected exception");
                    throw new CancelTransactionException("Intentional write request cancellation");
                }
            });
        } catch (DatabaseException e) {
            if (DEBUG)
                e.printStackTrace();
        }

        // Make sure that we can at least start a new transaction after the
        // cancelled one, but don't do anything yet.
        getSession().syncRequest(new WriteRequest() {
            @Override
            public void perform(WriteGraph graph) throws DatabaseException {
            }
        });

        // Ensure that the previously written data does not exist.
        getSession().syncRequest(new WriteRequest() {
            @Override
            public void perform(WriteGraph graph) throws DatabaseException {
                Layer0 b = Layer0.getInstance(graph);
                assertTrue(!graph.hasStatement(written, b.InstanceOf, b.Entity));
                assertTrue(!graph.hasStatement(written, b.HasName, writtenName));
                assertTrue(!graph.hasStatement(writtenName, b.InstanceOf, b.String));
                assertTrue(!graph.hasValue(writtenName));
            }
        });
    }

}
