/*******************************************************************************
 * Copyright (c) 2007, 2010 Association for Decentralized Information Management
 * in Industry THTH ry.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.db.tests.client;

import java.util.Collection;

import org.junit.Test;
import org.simantics.db.ReadGraph;
import org.simantics.db.Resource;
import org.simantics.db.Session;
import org.simantics.db.WriteGraph;
import org.simantics.db.common.request.WriteRequest;
import org.simantics.db.exception.DatabaseException;
import org.simantics.db.exception.ValidationException;
import org.simantics.db.service.LifecycleSupport;
import org.simantics.db.testing.base.ExistingDatabaseTest;
import org.simantics.db.testing.common.Tests;
import org.simantics.db.testing.impl.Configuration;
import org.simantics.layer0.Layer0;

public class ConnectionTest1 extends ExistingDatabaseTest {
//	private static final  String USERNAME = Configuration.get().username;
//	private static final String PASSWORD = Configuration.get().password;
	private static final int RECONNECT_COUNT = Configuration.get().connectionReconnectCount;

    @Test
	public void test1() throws DatabaseException {
	    Session session = null;
        for (int i=0; i<RECONNECT_COUNT; ++i) {
            connectionTest(session);
        }
	}
	void connectionTest(Session session) throws DatabaseException {
        try {
            session = Tests.getTestHandler().getSession();
            boolean TEST_QUERY = true;
            if (TEST_QUERY) {
                final String newInstanceName ="ConnectionTest1InstanceName-" +
                    getRandomString();
                session.syncRequest(new WriteRequest() {
                    @Override
                    public void perform(WriteGraph g) throws DatabaseException {
                        Layer0 b = Layer0.getInstance(g);
                        Resource newResource = g.newResource();
                        g.claim(newResource, b.InstanceOf, null, b.Type);
                        g.claimLiteral(newResource, b.HasName, newInstanceName);
                        g.claim(g.getRootLibrary(), b.ConsistsOf, newResource);
                    }
                });

                session.syncRequest(new TestReadRequest() {
                    @Override
                    public void run(ReadGraph g) throws DatabaseException {
                        Layer0 b = Layer0.getInstance(g);
                        Collection<Resource> resources = g.getObjects(g.getRootLibrary(), b.ConsistsOf);
                        Resource newResource = null;
                        for (Resource r : resources) {
                            String value = g.getPossibleRelatedValue(r, b.HasName);
                            if (null != value && value.equals(newInstanceName)) {
                                newResource = r;
                                break;
                            }
                        }
                        if (newResource == null) {
                            throw new ValidationException("Could not find created resource");
                        }
                        if (!g.isInstanceOf(newResource, b.Type))
                            throw new ValidationException("Created resource is not an instance of Type");
                    }
                });
            }
            LifecycleSupport ls = session.getService(LifecycleSupport.class);
            ls.close();
//        } catch (ClassNotFoundException e) {
//            throw new DatabaseException("Test failed.", e);
        } finally {
        }
	}
}
