/*******************************************************************************
 * Copyright (c) 2007, 2010 Association for Decentralized Information Management
 * in Industry THTH ry.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.db.tests.regression.bugs;

import java.util.Collection;
import java.util.Map;

import org.junit.Test;
import org.simantics.db.ChangeSetIdentifier;
import org.simantics.db.ReadGraph;
import org.simantics.db.Session;
import org.simantics.db.WriteGraph;
import org.simantics.db.common.CommentMetadata;
import org.simantics.db.common.UndoMetadata;
import org.simantics.db.common.request.ReadRequest;
import org.simantics.db.common.request.WriteRequest;
import org.simantics.db.exception.DatabaseException;
import org.simantics.db.service.ManagementSupport;
import org.simantics.db.service.XSupport;
import org.simantics.db.testing.base.ExistingDatabaseTest;

/**
 * Tests that commit which changes only meta data will create revision.
 */
public class Issue3072Test1 extends ExistingDatabaseTest {
    final boolean DEBUG = true;
    final String resourceName = "Issue3072Test1";
    // Behavior of commit has been changed. Commits with only CommentMetadata are dropped.
    private void addMetadata(WriteGraph wg, String s) throws DatabaseException {
        CommentMetadata cm = wg.getMetadata(CommentMetadata.class);
        wg.addMetadata(cm.add(s));
        wg.addMetadata(new UndoMetadata(null, false, 0, 0));
    }
	@Test
    public void testIssue3072Test1() throws Exception {
    	final Session session = getSession();
    	final ManagementSupport ms = session.getService(ManagementSupport.class);
    	final long revision1 = ms.getHeadRevisionId();
        session.syncRequest(new WriteRequest() {
            @Override
            public void perform(WriteGraph graph) throws DatabaseException {
                addMetadata(graph, "New comment.");
            }
        });
        final long revision2 = ms.getHeadRevisionId();
        assertTrue("Failed to create metadata only revision.", revision1 + 1 == revision2);
        session.syncRequest(new ReadRequest() {
            @Override
            public void run(ReadGraph graph) throws DatabaseException {
                Collection<ChangeSetIdentifier> cids = ms.getChangeSetIdentifiers(revision2, revision2);
                assertTrue("Failed to get empty change set.", cids.size() == 1);
                Map<String, byte[]> metadata = cids.iterator().next().getMetadata();
                assertTrue("Failed to get metadata.", metadata.size() > 0);
                XSupport xs = (XSupport)graph.getService(XSupport.class);
                xs.clearMetadataCache();
                Collection<ChangeSetIdentifier> cids2 = ms.getChangeSetIdentifiers(revision2, revision2);
                assertTrue("Failed to get empty change set.", cids2.size() == 1);
                Map<String, byte[]> metadata2 = cids2.iterator().next().getMetadata();
                assertTrue("Fail ed to get metadata.", metadata2.size() > 0);
            }
        });
    }
}
