/*******************************************************************************
 * Copyright (c) 2007, 2010 Association for Decentralized Information Management
 * in Industry THTH ry.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
import java.util.Collection;

import org.simantics.databoard.Bindings;
import org.simantics.db.ReadGraph;
import org.simantics.db.Resource;
import org.simantics.db.Session;
import org.simantics.db.WriteGraph;
import org.simantics.db.common.request.ReadRequest;
import org.simantics.db.common.request.WriteRequest;
import org.simantics.db.exception.DatabaseException;
import org.simantics.db.exception.ValidationException;
import org.simantics.db.testing.common.Tests;
import org.simantics.layer0.Layer0;


public class AdvancedHeadless {
    static String newInstanceName ="AdvancedClientTestInstanceName";
    static Session session = null;
    static Resource newResource;

    public static void main(String[] args) throws InterruptedException {
        try {
            session = Tests.getTestHandler().getSession();
            session.syncRequest(new WriteRequest() {
                @Override
                public void perform(WriteGraph g) throws DatabaseException {
                    Layer0 b = Layer0.getInstance(g);
                    newResource = g.newResource();
                    g.claim(newResource, b.InstanceOf, null, b.Type);
                    g.claimLiteral(newResource, b.HasName, newInstanceName);
                    g.claim(g.getRootLibrary(), b.ConsistsOf, newResource);
                    g.claimLiteral(newResource, b.HasDescription, "1");
                }
            });
            session.syncRequest(new WriteRequest() {

                @Override
                public void perform(WriteGraph g) throws DatabaseException {
                     Layer0 b = Layer0.getInstance(g);
                     g.claimLiteral(newResource, b.HasDescription, "11");

                }

            });

//            Thread.sleep(5000);

            session.syncRequest(new WriteRequest() {

                @Override
                public void perform(WriteGraph g) throws DatabaseException {
                     Layer0 b = Layer0.getInstance(g);
                     g.claimLiteral(newResource, b.HasDescription, "111");

                }

            });

//            Thread.sleep(5000);

            session.syncRequest(new WriteRequest() {

                @Override
                public void perform(WriteGraph g) throws DatabaseException {
                    Layer0 b = Layer0.getInstance(g);
                     g.claimLiteral(newResource, b.HasDescription, "1111");

                }

            });

            session.syncRequest(new ReadRequest() {
                @Override
                public void run(ReadGraph g) throws DatabaseException {
                    Layer0 b = Layer0.getInstance(g);
                    Collection<Resource> resources = g.getObjects(g.getRootLibrary(), b.ConsistsOf);
                    Resource newResource = null;
                    for (Resource r : resources) {
                        String value = g.getPossibleRelatedValue(r, b.HasName);
                        if (null != value && value.equals(newInstanceName)) {
                            newResource = r;
                            String s = g.getRelatedValue(r, b.HasDescription, Bindings.STRING);
                            if (!s.equals("1111"))
                                throw new ValidationException("Value is not correct.");
                            break;
                        }
                    }
                    if (newResource == null) {
                        throw new ValidationException("Could not find created resource");
                    }
                    if (!g.isInstanceOf(newResource, b.Type))
                        throw new ValidationException("Created resource is not an instance of Type");
                }
            });

            session.syncRequest(new ReadRequest() {
                @Override
                public void run(ReadGraph g) throws DatabaseException {
                    Layer0 b = Layer0.getInstance(g);
                    int count = 0;
                    Collection<Resource> resources = g.getObjects(g.getRootLibrary(), b.ConsistsOf);
                    for (Resource r : resources) {
                        String value = g.getPossibleRelatedValue(r, b.HasName);
                        if (null != value && value.equals(newInstanceName)) {
                            count++;
                        }
                    }
                    System.out.println("instance count = " + count);
                }
            });

            //LifecycleSupport ls = session.getService(LifecycleSupport.class);
            //ls.save();
            //ls.close();
            System.out.println("Ok. No ls.close()");
            System.exit(0);

//        } catch (ClassNotFoundException e) {
//            e.printStackTrace();
        } catch (DatabaseException e) {
            e.printStackTrace();
        }
    }
}
